<?php

declare(strict_types=1);

namespace Dedi\SyliusSogenactifPlugin\Payum\Sogenactif\Action;

use Payum\Core\Action\ActionInterface;
use Payum\Core\Bridge\Spl\ArrayObject;
use Payum\Core\Exception\RequestNotSupportedException;
use Payum\Core\GatewayAwareInterface;
use Payum\Core\GatewayAwareTrait;
use Payum\Core\GatewayInterface;
use Payum\Core\Request\GetHttpRequest;
use Payum\Core\Request\GetStatusInterface;

final class Status implements ActionInterface, GatewayAwareInterface
{
    use GatewayAwareTrait;

    public function execute($request): void
    {
        RequestNotSupportedException::assertSupports($this, $request);

        $this->gateway->execute($httpRequest = new GetHttpRequest());

        $dataRequest = $httpRequest->request['Data'] ?? null;

        if (null === $dataRequest) {
            $request->markNew();

            return;
        }

        $details = $this->getDetailsFromRequest($dataRequest);

        $paymentDetails = ArrayObject::ensureArrayObject($request->getModel());
        $paymentDetails->replace($details);

        $request->setModel($paymentDetails);

        if (count($details) > 0 &&
            array_key_exists('all_normalized_infos', $details) &&
            array_key_exists('responseCode', $details['all_normalized_infos']) &&
            '00' === $details['all_normalized_infos']['responseCode']
        ) {
            $request->markCaptured();

            return;
        }

        $request->markFailed();
    }

    public function supports($request): bool
    {
        return
            $request instanceof GetStatusInterface &&
            $request->getModel() instanceof \ArrayAccess
        ;
    }

    public function setGateway(GatewayInterface $gateway): void
    {
        $this->gateway = $gateway;
    }

    private function getDetailsFromRequest(mixed $dataRequest): array
    {
        $datas = explode('|', $dataRequest);

        $normalizeData = [];
        $details = [];

        foreach ($datas as $data) {
            if (strpos($data, '=')) {
                $explodedData = explode('=', $data);
                if (array_key_exists(0, $explodedData) && array_key_exists(1, $explodedData)) {
                    $normalizeData[$explodedData[0]] = $explodedData[1];
                } else {
                    $details['error_data_from_return'][] = 'Key value pair for normalized data is null for data receive ' . $data;
                }
            } else {
                $details['error_data_from_return'][] = 'No data, maybe bad separator. Should be like `Key=Value|Key=Value` but data receive is ' . $data;
            }
        }

        $details['request_data_from_payback'] = $dataRequest;
        $details['all_normalized_infos'] = $normalizeData;

        return $details;
    }
}
